package cc.fascinated.bat.features.base.commands.botadmin.premium;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.premium.PremiumProfile;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "remove", description = "Remove premium from a guild")
public class RemoveSubCommand extends BatCommand {
    private final GuildService guildService;

    @Autowired
    public RemoveSubCommand(GuildService guildService) {
        this.guildService = guildService;
        super.addOptions(new OptionData(OptionType.STRING, "guild", "The guild id to set as premium", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping guildOption = event.getOption("guild");
        if (guildOption == null) {
            event.reply("Please provide a guild id").queue();
            return;
        }
        String guildId = guildOption.getAsString();
        BatGuild targetGuild = guildService.getGuild(guildId);
        if (targetGuild == null) {
            event.reply("The guild with the id %s does not exist".formatted(guildId)).queue();
            return;
        }
        PremiumProfile premium = targetGuild.getPremiumProfile();
        if (!premium.hasPremium()) {
            event.reply("The guild does not have premium").queue();
            return;
        }

        premium.removePremium();
        event.reply("The guild **%s** has had its premium removed".formatted(targetGuild.getName())).queue();
    }
}
