package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:list.sub")
@CommandInfo(name = "list", description = "See all the log types and their channels")
public class ListSubCommand extends BatCommand implements EventListener {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        List<Button> buttons = new ArrayList<>();
        buttons.add(Button.primary("logs-home", "Home").withEmoji(Emojis.HOME_EMOJI));
        for (LogCategory category : LogCategory.values()) {
            buttons.add(Button.primary("logs:list:%s".formatted(category.name().toLowerCase()), category.getName()).withEmoji(category.getEmoji()));
        }
        event.replyEmbeds(getHelpMessage()).addComponents(ActionRow.of(buttons)).queue();
    }

    @Override
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        if (guild == null || event.getMember() == null) {
            return;
        }

        // Not a log button
        if (!event.getComponentId().startsWith("logs")) {
            return;
        }

        // No permissions
        if (!event.getMember().hasPermission(this.getInfo().getPermissions())) {
            event.reply("%s, you cannot use this button.".formatted(user.getDiscordUser().getAsMention()))
                    .setEphemeral(true)
                    .queue();
            return;
        }

        // Home
        if (event.getComponentId().equals("logs-home")) {
            event.editMessageEmbeds(getHelpMessage()).queue();
            return;
        }

        // Category
        String[] split = event.getComponentId().split(":");
        if (split.length != 3) {
            return;
        }
        LogProfile profile = guild.getLogProfile();
        LogCategory category = LogCategory.getLogCategory(split[2].toUpperCase());
        if (category == null) {
            return;
        }

        List<LogType> logEvents = LogType.getLogTypesByCategory(category.getName());
        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder(null);
        description.appendLine("Log channels for the `%s` category".formatted(category.getName()), false);
        description.emptyLine();
        for (LogType logType : logEvents) {
            if (logType.getCategory() != category) {
                continue;
            }
            TextChannel logChannel = profile.getLogChannel(logType);
            description.appendLine("%s: %s".formatted(logType.getName(), logChannel == null ? "Not Set" : logChannel.getAsMention()), true);
        }
        event.editMessageEmbeds(EmbedUtils.genericEmbed().setDescription(description.build()).build()).queue();
    }

    /**
     * Gets the help message for the command
     *
     * @return the help message
     */
    public MessageEmbed getHelpMessage() {
        return EmbedUtils.genericEmbed().setDescription(new EmbedDescriptionBuilder(null).appendLine(
                """
                        Set the log channel for:
                        - A specific event, use `/logs set <event> <channel>`
                        - A specific category by using `/logs set <category> <channel>`
                        - All log events by using `/logs set all <channel>`
                        To remove a log channel, it's the same as setting it,
                        but with `/logs remove` instead of `/logs set`
                        \s
                        *Use the buttons below to see the log events for each category*
                        """, false).build()).build();
    }
}

