package cc.fascinated.bat.model;

import cc.fascinated.bat.common.ChannelUtils;
import cc.fascinated.bat.common.UserUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.springframework.data.redis.core.RedisHash;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter @Setter
@RedisHash(value = "DiscordMessage", timeToLive = 86400) // 24 hours
public class DiscordMessage {
    /**
     * The snowflake ID of the message
     */
    private final String id;

    /**
     * The timestamp when the message was sent
     */
    private final long timestamp;
    
    /**
     * The snowflake ID of the channel the message was sent in
     */
    private final String channelId;
    
    /**
     * The snowflake ID of the guild the message was sent in
     */
    private final String guildId;
    
    /**
     * The snowflake ID of the author of the message
     */
    private final String authorId;
    
    /**
     * The content of the message
     */
    private String content;

    /**
     * Whether the message was deleted
     */
    private boolean deleted;

    /**
     * Gets the author of the message
     *
     * @return the author
     */
    public User getAuthor() {
        return UserUtils.getUser(this.authorId);
    }

    /**
     * Gets the channel the message was sent in
     *
     * @return the channel
     */
    public TextChannel getChannel() {
        return ChannelUtils.getTextChannel(this.channelId);
    }

    /**
     * Gets the URL of the message
     *
     * @return the URL
     */
    public String getMessageUrl() {
        return "https://discord.com/channels/%s/%s/%s".formatted(this.guildId, this.channelId, this.id);
    }
}
